@@ -1,10 +1,9 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
+
 module TextBuilderDev.TastyExtras where
 
-import qualified Data.Text as Text
 import qualified Data.Text.Lazy.Builder as TextLazyBuilder
-import Test.QuickCheck.Instances
 import Test.Tasty
-import Test.Tasty.HUnit
 import Test.Tasty.QuickCheck
 import qualified TextBuilderDev as B
 import Prelude hiding (choose)
@@ -15,10 +14,6 @@ instance Arbitrary TextLazyBuilder.Builder where
   arbitrary =
     TextLazyBuilder.fromLazyText <$> arbitrary
 
-instance Arbitrary B.TextBuilder where
-  arbitrary =
-    B.lazyText <$> arbitrary
-
 -- * --
 
 isomorphismLaws ::
@@ -27,13 +22,16 @@ isomorphismLaws ::
   Proxy a ->
   TestTree
 isomorphismLaws subject proxy =
-  testGroup subject $
-    [ testProperty "fromTextBuilder . toTextBuilder == id" $
-        (===)
-          <$> B.fromTextBuilder . B.toTextBuilder
+  testGroup subject
+    $ [ testProperty "fromTextBuilder . toTextBuilder == id"
+          $ (===)
+          <$> B.fromTextBuilder
+          . B.toTextBuilder
           <*> flip asProxyTypeOf proxy,
-      testProperty "toTextBuilder . fromTextBuilder == id" $
-        (===)
-          <$> B.toTextBuilder . flip asProxyTypeOf proxy . B.fromTextBuilder
+        testProperty "toTextBuilder . fromTextBuilder == id"
+          $ (===)
+          <$> B.toTextBuilder
+          . flip asProxyTypeOf proxy
+          . B.fromTextBuilder
           <*> id
-    ]
+      ]
